unit uMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Spin, ComCtrls, wrapi;

type
  TMainForm = class(TForm)
    RadioPanel: TPanel;
    DSPPanel: TPanel;
    Label1: TLabel;
    FileNameEdit: TEdit;
    RecordButton: TButton;
    StopButton: TButton;
    DeviceNumberSpinEdit: TSpinEdit;
    Label2: TLabel;
    Label3: TLabel;
    HandleLabel: TLabel;
    OpenButton: TButton;
    CloseButton: TButton;
    Label4: TLabel;
    FrequencyEdit: TEdit;
    Label5: TLabel;
    VolumeTrackBar: TTrackBar;
    Label6: TLabel;
    AMButton: TButton;
    FMNButton: TButton;
    FMWButton: TButton;
    LSBButton: TButton;
    USBButton: TButton;
    FMMButton: TButton;
    PowerCheckBox: TCheckBox;
    SSGauge: TProgressBar;
    RAWGauge: TProgressBar;
    Label7: TLabel;
    Label8: TLabel;
    SSTimer: TTimer;
    Label9: TLabel;
    BlockSizeLabel: TLabel;
    BuffersLabel: TLabel;
    Label12: TLabel;
    BytesWrittenLabel: TLabel;
    Label10: TLabel;
    PlayButton: TButton;
    Label11: TLabel;
    BytesReadLabel: TLabel;
    Scope: TPanel;
    procedure OpenButtonClick(Sender: TObject);
    procedure SSTimerTimer(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure VolumeTrackBarChange(Sender: TObject);
    procedure FrequencyEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure AMButtonClick(Sender: TObject);
    procedure LSBButtonClick(Sender: TObject);
    procedure USBButtonClick(Sender: TObject);
    procedure FMNButtonClick(Sender: TObject);
    procedure FMWButtonClick(Sender: TObject);
    procedure FMMButtonClick(Sender: TObject);
    procedure PowerCheckBoxClick(Sender: TObject);
    procedure RecordButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure PlayButtonClick(Sender: TObject);
  private
    { Private declarations }
    Procedure WMDSPAckRead(var Message: TRMDMAAckRead); Message RM_DSPACKREAD;
    Procedure WMDSPAckWrite(var Message: TRMDMAAckWrite); Message RM_DSPACKWRITE;
    Procedure BufferToScope(Buf : PChar; Size : Integer); 
  public
    { Public declarations }
    RadioInfo : TRadioInfo;
    hRadio : Integer;
    BlockSize : Integer;
    Buffers : Integer;
    BytesWritten : Integer;
    BytesRead : Integer;
    OutFile : THandle;
    ClosingDSP : Bool;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.OpenButtonClick(Sender: TObject);
var
  f : Int64;
begin
  hRadio := OpenRadioDevice(DeviceNumberSpinEdit.Value);
  HandleLabel.Caption := IntToStr(hRadio);

  if (hRadio > 0) then
  begin
    OpenButton.Enabled := False;
    CloseButton.Enabled := True;

    RadioInfo.dwSize := SizeOf(TRadioInfo);
    GetRadioDeviceInfo(hRadio, @RadioInfo);
    if (RadioInfo.dwFeatures and RIF_DSP <> 0) then
      DSPPanel.Enabled := True;

    f := GetFrequency(hRadio);
    if (f and RFQ_X10 <> 0) then
    begin
      f := f xor RFQ_X10;
      f := f * 10;
    end;
    FrequencyEdit.Text := IntToStr(f);

    VolumeTrackBar.Min := 0;
    VolumeTrackBar.Max := RadioInfo.iMaxVolume;
    VolumeTrackBar.Position := GetVolume(hRadio);

    SetPower(hRadio, True);
    PowerCheckBox.Checked := True;

    SSTimer.Enabled := True;
  end;
end;

procedure TMainForm.SSTimerTimer(Sender: TObject);
begin
  SSGauge.Position := GetSignalStrength(hRadio);
  RAWGauge.Position := GetRawSignalStrength(hRadio);
end;

procedure TMainForm.CloseButtonClick(Sender: TObject);
begin
  SSTimer.Enabled := False;

  OpenButton.Enabled := True;
  CloseButton.Enabled := False;

  HandleLabel.Caption := '0';

  if (OutFile <> 0) then
    StopButtonClick(Sender);

  CloseRadioDevice(hRadio);

  hRadio := 0;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if (hRadio > 0) then
    CloseButtonClick(Sender);
end;

procedure TMainForm.VolumeTrackBarChange(Sender: TObject);
begin
  SetVolume(hRadio, VolumeTrackBar.Position);
end;

procedure TMainForm.FrequencyEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  f1 : Int64;
  f2 : Integer;
begin
  if (Key = VK_RETURN) then
  begin
    f1 := 0;
    try
      f1 := StrToInt(FrequencyEdit.Text);
    finally
      if (f1 > MaxInt) then
      begin
        f2 := f1 div 10;
        f2 := f2 or RFQ_X10;
      end
      else
        f2 := f1;
      SetFrequency(hRadio, f2);
    end;
  end;
end;

procedure TMainForm.AMButtonClick(Sender: TObject);
begin
  SetMode(hRadio, RMD_AM);
end;

procedure TMainForm.LSBButtonClick(Sender: TObject);
begin
  SetMode(hRadio, RMD_LSB);
end;

procedure TMainForm.USBButtonClick(Sender: TObject);
begin
  SetMode(hRadio, RMD_USB);
end;

procedure TMainForm.FMNButtonClick(Sender: TObject);
begin
  SetMode(hRadio, RMD_FMN);
end;

procedure TMainForm.FMWButtonClick(Sender: TObject);
begin
  SetMode(hRadio, RMD_FMW);
end;

procedure TMainForm.FMMButtonClick(Sender: TObject);
begin
  SetMode(hRadio, RMD_FMM);
end;

procedure TMainForm.PowerCheckBoxClick(Sender: TObject);
begin
  SetPower(hRadio, PowerCheckBox.Checked);
end;

procedure TMainForm.RecordButtonClick(Sender: TObject);
var
  hdr : PDSPHdr;
  i : Integer;
begin
  OutFile := CreateFile(PChar(FileNameEdit.Text),
                        GENERIC_WRITE,
                        FILE_SHARE_READ,
                        nil,
                        CREATE_ALWAYS,
                        FILE_ATTRIBUTE_NORMAL,
                        0);

  if (OutFile <> INVALID_HANDLE_VALUE) then
  begin
    RecordButton.Enabled := False;
    PlayButton.Enabled := False;
    StopButton.Enabled := True;
    BytesWritten := 0;
    ClosingDSP := False;
    SetDSPSource(hRadio, RDS_RADIO);
    BlockSize := dspInOpen(hRadio, RWF_8kHz or RWF_16bit or RWF_Mono, MainForm.Handle);
    BlockSizeLabel.Caption := IntToStr(BlockSize);
    for i := 1 to 10 do
    begin
      New(hdr);
      GetMem(hdr^.lpData, BlockSize);
      hdr^.dwBufferLength := BlockSize;
      hdr^.dwBytesTransferred := 0;
      hdr^.dwUser := 0;
      hdr^.dwFlags := 0;
      hdr^.dwDateTransferred := 0;
      hdr^.dwTimeTransferred := 0;
      dspInAddBuffer(hRadio, hdr, SizeOf(TDSPHdr));
      Inc(Buffers);
      BuffersLabel.Caption := IntToStr(Buffers);
    end;
  end
  else begin
    Application.MessageBox(PChar('Error opening file: ' + FileNameEdit.Text+#10#13+'Error code: '+IntToStr(GetLastError)), 'ERROR', MB_OK);
    OutFile := 0;
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  hRadio := 0;
  OutFile := 0;
  ClosingDSP := True;
end;

Procedure TMainForm.WMDSPAckRead(var Message: TRMDMAAckRead);
var
  x : Cardinal;
begin
  dspBufferDone(Message.hRadio, Message.lpDSPHdr, SizeOf(TDSPHdr));
  BufferToScope(Message.lpDSPHdr.lpData, Message.lpDSPHdr.dwBytesTransferred);

  if (not ClosingDSP) then
  begin
    WriteFile(OutFile, Message.lpDSPHdr.lpData^, Message.lpDSPHdr^.dwBytesTransferred, x, nil);
    Inc(BytesWritten, x);
    BytesWrittenLabel.Caption := IntToStr(BytesWritten);
    dspInAddBuffer(Message.hRadio, Message.lpDSPHdr, SizeOf(TDSPHdr));
  end
  else begin
    FreeMem(Message.lpDSPHdr^.lpData);
    FreeMem(Message.lpDSPHdr);
    Dec(Buffers);
    BuffersLabel.Caption := IntTOStr(Buffers);
  end;
end;

Procedure TMainForm.WMDSPAckWrite(var Message: TRMDMAAckWrite);
var
     x : Cardinal;
begin
  dspBufferDone(Message.hRadio, Message.lpDSPHdr, SizeOf(TDSPHdr));
  BufferToScope(Message.lpDSPHdr.lpData, Message.lpDSPHdr.dwBufferLength);

  if (not ClosingDSP) then
  begin
    ReadFile(OutFile, Message.lpDSPHdr.lpData^, BlockSize, x, nil);
    if (x = 0)
    then begin
      // end of file
      // dispatch this buffer
      FreeMem(Message.lpDSPHdr^.lpData);
      FreeMem(Message.lpDSPHdr);
      Dec(Buffers);
      BuffersLabel.Caption := IntTOStr(Buffers);
      // simulate mouse click on Stop Button
      StopButton.Click;
    end
    else begin
      Inc(BytesRead, x);
      BytesReadLabel.Caption := IntToStr(BytesRead);
      Message.lpDSPHdr.dwBufferLength := x;
      dspOutWrite(Message.hRadio, Message.lpDSPHdr, SizeOf(TDSPHdr));
    end;
  end
  else begin
    FreeMem(Message.lpDSPHdr^.lpData);
    FreeMem(Message.lpDSPHdr);
    Dec(Buffers);
    BuffersLabel.Caption := IntTOStr(Buffers);
  end;
end;

procedure TMainForm.StopButtonClick(Sender: TObject);
begin
  if (not ClosingDSP) then
  begin
    ClosingDSP := True;
    MainForm.Enabled := False;

    repeat
      Application.ProcessMessages;
    until Buffers = 0;

    dspClose(hRadio);
    SetAudioSource(hRadio, RAS_RADIO);
    CloseHandle(OutFile);
    OutFile := 0;

    RecordButton.Enabled := True;
    PlayButton.Enabled := True;
    StopButton.Enabled := False;

    MainForm.Enabled := True;
  end;
end;

procedure TMainForm.PlayButtonClick(Sender: TObject);
var
     hdr : PDSPHdr;
     i : Integer;
     x : Cardinal;
begin
  OutFile := CreateFile(PChar(FileNameEdit.Text),
                        GENERIC_READ,
                        FILE_SHARE_READ,
                        nil,
                        OPEN_EXISTING,
                        FILE_ATTRIBUTE_NORMAL,
                        0);

  if (OutFile <> INVALID_HANDLE_VALUE) then
  begin
    RecordButton.Enabled := False;
    PlayButton.Enabled := False;
    StopButton.Enabled := True;
    BytesRead := 0;
    ClosingDSP := False;
    SetAudioSource(hRadio, RAS_DSP);
    BlockSize := dspOutOpen(hRadio, RWF_8kHz or RWF_16bit or RWF_Mono, MainForm.Handle);
    BlockSizeLabel.Caption := IntToStr(BlockSize);
    for i := 1 to 10 do
    begin
      New(hdr);
      GetMem(hdr^.lpData, BlockSize);
      ReadFile(OutFile, hdr^.lpData^, BlockSize, x, nil);
      Inc(BytesRead, x);
      if (x = 0)
      then begin
        FreeMem(hdr^.lpData);
        FreeMem(hdr);
        break;
      end;
      hdr^.dwBufferLength := BlockSize;
      hdr^.dwBytesTransferred := 0;
      hdr^.dwUser := 0;
      hdr^.dwFlags := 0;
      hdr^.dwDateTransferred := 0;
      hdr^.dwTimeTransferred := 0;
      dspOutWrite(hRadio, hdr, SizeOf(TDSPHdr));
      Inc(Buffers);
      BuffersLabel.Caption := IntToStr(Buffers);
    end;
  end
  else begin
    Application.MessageBox(PChar('Error opening file: ' + FileNameEdit.Text+#10#13+'Error code: '+IntToStr(GetLastError)), 'ERROR', MB_OK);
    OutFile := 0;
  end;
end;

Procedure TMainForm.BufferToScope(Buf : PChar; Size : Integer);
var
     i,n : Integer;
     x : SmallInt;
     c : TControlCanvas;
begin
  if (Size > Scope.Width - 4)
    then n := Scope.Width - 4
    else n := Size;

  c := TControlCanvas.Create;
  c.Control := Scope;

  c.Brush.Color := clBlack;
  c.Rectangle(2, 2, Scope.Width - 2, Scope.Height - 2);
  c.Pen.Color := clGreen;
  c.MoveTo(2, (Scope.Height - 4) div 2);
  c.LineTo(Scope.Width - 2, (Scope.Height - 4) div 2);
  c.Pen.Color := clWhite;
  move(Buf^, x, 2);
  c.MoveTo(2, x div 700 + (Scope.Height - 4) div 2);

  for i := 1 to n do
  begin
    move((Buf + i*2)^, x, 2);
    c.LineTo(i + 2, x div 700 + (Scope.Height - 4) div 2);
  end;

  c.Destroy;
end;

end.
